clear
close
clc
addpath(genpath('src'))
rng(1)

% ============
% Beam example
% ============
A = beam();
% We will look for approximations of the eigenvalues with accuracy 10^{-13}
tol = 1e-12;
% Domain of interest is on the real part between 100 and 30000, with
% positive imaginary part. We use 10000 points.
% set_aaa = [(rand(1e4,1)*(30000-200)+200) + 1i*rand(1e4,1)*1000+1e-4;linspace(200,30000,1e4).'];
set_aaa = [linspace(200,30000,1e4).'];
set_test = 200 + rand(1000,1)*(30000-200);
f = A.f{1}(set_test);
nf = norm(f);
n_pol = zeros(1,15); err = zeros(1,15);
for d = 2:15
    [r,pol] = aaa_sv(A.f{1},set_aaa.','tol',tol,'mmax',d);
    n_pol(d-1) = length(pol);
    err(d-1) = norm(r(set_test)-f)/nf;
end
% Get the eigenvalues of the non-linear matrix function, for a test set
% between branchpoint 0 and branchpoint 7
opts = struct('m',70,'shifts',[200,500,1000,10000,20000,21000,22000,23000,24000,25000],'tol',tol);
[~,~,~,info_aaa] = eign(A,set_aaa,opts);

% Postprocessing of the residues and Ritz values
[ lambda_sor,res_sor ] = sort_eig( info_aaa.Lambda,info_aaa.Res );
is_in = logical((real(lambda_sor(:,end))>=200).*(real(lambda_sor(:,end))<=25000));
[vals,inds] = sort(res_sor(is_in,end));
ritz = lambda_sor(is_in,end); ritz = ritz(inds(1:8));

figure
plot(log10(real(ritz)),log10(imag(ritz)),'x','color','red')
hold on
plot(-log10(-real(pol)),log10(imag(pol)),'o','color',[0,0,0.5])
xlabel('sign($\lambda$) $log_{10} (|Re \lambda|)$','Interpreter','Latex','Fontsize',12)
ylabel('$log_{10}(Im \lambda$)','Interpreter','Latex','Fontsize',12)
legend('Ritz values','Branch points','Location','NorthWest')
title('Ritz values and first eight branch points.','Fontsize',14,'Interpreter','Latex')

% Make figures
figure
semilogy(n_pol,err,'-o','color',[0,0,0.5])
ylabel('Error ($E$)','Interpreter','Latex','Fontsize',12)
xlabel('Number of poles ($m$)','Interpreter','Latex','Fontsize',12)

figure
semilogy(res_sor(is_in,:).','-','color',[0,0,0.5])
hold on
semilogy(res_sor(~is_in,:).',':','color',[0,0,0.5])
xlabel('iteration $j$','Interpreter','Latex','Fontsize',12)
ylabel('$R_x$','Interpreter','Latex','Fontsize',12)
title('Residue for the two Ritz values','Fontsize',14,'Interpreter','Latex')