clear
close
clc
rng(1)
addpath(genpath('src'))

% ==============
% Canyon example
% ==============
[A,brpts] = canyon();
f_v = @(x) cell2mat(cellfun(@(fi) feval(fi,x),A.f,'UniformOutput',false));
% We will look for approximations of the eigenvalues with accuracy 10^{-13}
tol = 1e-13;
% Domain of interest is between branch point 0 and branchpoint 1-7,
% test set consists of 1000 points.
n_pol = zeros(1,7);
n_pol_sv = zeros(1,7);
for bp = 1:7
    for it = 1:length(A.f)
        [~,pol] = aaa_sv(A.f{it},linspace(brpts(1),brpts(bp+1),2000).','tol',tol,'mmax',500);
        n_pol(bp) = n_pol(bp) + length(pol);
    end
    [~,pol] = aaa_sv(f_v,linspace(brpts(1),brpts(bp+1),2000).','tol',tol,'mmax',500);
    n_pol_sv(bp) = length(pol);
end
% Get the eigenvalues of the non-linear matrix function, for a test set
% between branchpoint 0 and branchpoint 7
opts = struct('m',160,'shifts',linspace(brpts(1)+1e-2,brpts(2)-1e-2,5),'tol',tol,'scale_res',false,'pos_im',true);
[~,~,~,info_aaa] = eign(A,linspace(brpts(1),brpts(8),2000),opts);

% Postprocessing of the residues and Ritz values
[ lambda_sor,res_sor ] = sort_eig( info_aaa.Lambda,info_aaa.Res );
res_sor(res_sor == 0) = inf;
is_in = logical((real(lambda_sor(:,end))>=brpts(1)).*(real(lambda_sor(:,end))<=brpts(8)));
res = res_sor(is_in,:); res = res(res_sor(is_in,end) < 1e-9,:);
ritz = lambda_sor(is_in,end); ritz = ritz(res_sor(is_in,end) < 1e-9);

% Make figures
figure
plot(1:7,n_pol,'-s','color',[0,0.5,0.5])
hold on
plot(1:7,n_pol_sv,'-o','color',[0,0,0.5])
xlabel('end branch point ($\alpha$)','Interpreter','Latex','Fontsize',12)
ylabel('number of poles ($m$)','Interpreter','Latex','Fontsize',12)
legend('AAA','Set-valued AAA','location','NorthWest')
title('Approximation error','Fontsize',14,'Interpreter','Latex')

figure
plot(real(ritz),imag(ritz),'x','color',[0,0,0.5])
hold on
plot(real(brpts(1:8)),imag(brpts(1:8)),'o','color','red')
xlim([brpts(1)-0.1,brpts(8)+0.1])
ylim([-1,1])
xlabel('Re $\lambda$','Interpreter','Latex','Fontsize',12)
ylabel('Im $\lambda$','Interpreter','Latex','Fontsize',12)
legend('Ritz values','Branch points','Location','NorthWest')
title('Ritz values and first eight branch points.','Fontsize',14,'Interpreter','Latex')

figure
semilogy(res.','-','color',[0,0,0.5])
xlabel('iteration $j$','Interpreter','Latex','Fontsize',12)
ylabel('$R_x$','Interpreter','Latex','Fontsize',12)
title('Residue for the two Ritz values','Fontsize',14,'Interpreter','Latex')