clear
close
clc
rng(1)
addpath(genpath('src'))

% ==================
% Car cavity example
% ==================
% Load the car cavity problem
model = car();
ftions = @(x) cell2mat(cellfun(@(fi) feval(fi,x),model.A.f,'UniformOutput',false));

% 1. View the poles of the model
% ------------------------------
% 1.1 Set of interest upto first special point (514i)
% aaa consists of random points with positive imaginary part and real part ...
% in frequency range and points on the real axes in the
% frequency range of intrest
n_aaa = 1e5;
set_aaa = [(rand(n_aaa/2,1)*(model.f(end)-model.f(1))+model.f(1)) + 1i*rand(n_aaa/2,1)*500;linspace(model.f(1),model.f(end),n_aaa/2).'];
[~,pol_aaa_1] = aaa_sv(ftions,set_aaa,'tol',1e-13,'mmax',300);
% Leja bagby apprach has poles that start in the special points (514i) and Sigma
% running almost up to it, in a square bounded by the frequency domain and
% 0 (degree determined over test set equal to that of aaa)
n_lb = 4e5;
Xi = 1i*logspace(log10(514),6,n_lb);
Sigma = [linspace(1,300,n_lb/4),1+1i*linspace(0,510,n_lb/4),1i*linspace(0,510,n_lb/4)+300,linspace(1,300,n_lb/4)+1i*510];
% Degree 40 appears good (uncomment below to see automatic determination)
% err = inf;
% d = 1;
% f = ftions(set_aaa);
% while err > 1e-13 && d < 100
%     d = d+1;
%     [sigma_lb,xi_lb,beta_lb] = lejabagby(Sigma,Xi,Sigma,d,false,-1);
%     D = ratnewtoncoeffs(ftions,sigma_lb,xi_lb,beta_lb);
%     f_test = zeros(size(f));
%     for it_in = 1:d
%         erat = evalrat(sigma_lb(1:it_in-1),xi_lb(1:it_in-1),beta_lb(1:it_in),set_aaa);
%         f_test(:,1) = f_test(:,1) + D{it_in}(1)*erat;
%         f_test(:,2) = f_test(:,2) + D{it_in}(2)*erat;
%     end
%     err = max(max(abs(f-f_test)))/max(max(abs(f)));
% end
d = 40;
[~,xi_lb,~] = lejabagby(Sigma,Xi,Sigma,d,false,-1);
% Readjust the number of samples for plotting
n_aaa = 80;
set_aaa_1 = [(rand(n_aaa/2,1)*(model.f(end)-model.f(1))+model.f(1)) + 1i*rand(n_aaa/2,1)*510;linspace(model.f(1),model.f(end),n_aaa/2).'];
n_lb = 80;
set_lb = [linspace(1,300,n_lb/4),1+1i*linspace(0,510,n_lb/4),1i*linspace(0,510,n_lb/4)+300,linspace(1,300,n_lb/4)+1i*510];

figure('name','Poles and sets')
plot(real(set_aaa_1),imag(set_aaa_1),'x','Markersize',4)
hold on
plot(real(set_lb),imag(set_lb),'.')
plot(real(pol_aaa_1),imag(pol_aaa_1),'o')
plot(real(xi_lb),imag(xi_lb),'d','Markersize',4)
xlim([-200,310]);
ylim([-50,3000]);
xlabel('Re')
ylabel('Im')
title('Set and poles (not all poles visible)')
legend('Set AAA','Sigma',['Poles AAA (' num2str(length(pol_aaa_1)) ')'],['Poles L-B (' num2str(length(xi_lb)) ')'])

% 1.2 Set of interest for very large imaginary axis (aaa only)
% You can clearly see the four special points found as poles for aaa.
% Note that this requires more poles than before
n_aaa = 1e5;
set_aaa_2 = [(rand(n_aaa/2,1)*(model.f(end)-model.f(1))+model.f(1)) + 1i*rand(n_aaa/2,1)*10000;linspace(model.f(1),model.f(end),n_aaa/2).'];
[~,pol_aaa_2] = aaa_sv(ftions,set_aaa_2,'tol',1e-13,'mmax',300);
n_aaa = 500;
set_aaa_2 = [(rand(n_aaa/2,1)*(model.f(end)-model.f(1))+model.f(1)) + 1i*rand(n_aaa/2,1)*10000;linspace(model.f(1),model.f(end),n_aaa/2).'];

figure('name','Poles and sets, revised')
plot(real(set_aaa_2),imag(set_aaa_2),'x','Markersize',4)
hold on
plot(real(pol_aaa_2),imag(pol_aaa_2),'o')
xlim([-200,310]);
ylim([-50,3000]);
xlabel('Re')
ylabel('Im')
title('Set and poles for high i (not all poles visible)')
legend('Set AAA',['Poles AAA (' num2str(length(pol_aaa_2)) ')'])

% 2. Eigenvalue computation
% --------------------------
% 2.1 AAA
% -------
% Choosing the first test sets for finding the eigenvalues
n_aaa = 1e4;
sigma = [(rand(n_aaa/2,1)*(model.f(end)-model.f(1))+model.f(1)) + 1i*rand(n_aaa/2,1)*510;linspace(model.f(1),model.f(end),n_aaa/2).'];
% Take 10 equally spaced interpolation points in the frequency interval for the
% rational Krylov routine
opts.shifts = linspace(model.f(1),model.f(end),10);
% Number of iterations
opts.m = 150;
% Set two-sided algorithm
opts.v0 = rand(length(model.B),1);
% Get the eigenvalues
[~,~,~,info_aaa] = eign(model.A,sigma,opts);

% 2.2 Leja-Bagby
% --------------
opts.kind = 'ratNewton';
opts.d = 40;
n_lb = 4e5;
opts.Xi = 1i*logspace(log10(514),6,n_lb);
Sigma =  [linspace(1,300,n_lb/4),1+1i*linspace(0,510,n_lb/4),300 + 1i*linspace(0,510,n_lb/4),linspace(1,300,n_lb/4)+1i*510];
[~,~,~,info_Newton] = eign(model.A,Sigma,opts);

% Processing the data
[ lambda_sor_newton,res_sor_newton ] = sort_eig( info_Newton.Lambda,info_Newton.Res );
[ lambda_sor_aaa,res_sor_aaa ] = sort_eig( info_aaa.Lambda,info_aaa.Res );
is_in_newton = logical((real(lambda_sor_newton(:,end))>=min(real(opts.shifts))).*(real(lambda_sor_newton(:,end))<=max(real(opts.shifts))));
is_in_aaa = logical((real(lambda_sor_aaa(:,end))>=min(real(opts.shifts))).*(real(lambda_sor_aaa(:,end))<=max(real(opts.shifts))));

ritz_newton = lambda_sor_newton(is_in_newton,end);
ritz_aaa = lambda_sor_aaa(is_in_aaa,end);
res_newton = res_sor_newton(is_in_newton,:);
res_aaa = res_sor_aaa(is_in_aaa,:);
res_newton(res_newton==0) = inf;
res_aaa(res_aaa==0) = inf;
k_newton = zeros(1,min(length(ritz_newton),length(ritz_aaa)));
k_aaa = zeros(size(k_newton));
for it = 1:min(length(ritz_newton),length(ritz_aaa))
    [~,loc] = min(abs(ritz_newton(it)-ritz_aaa));
    ll_aaa = find(res_aaa(loc,:)<1e-12,1,'first');
    ll_newton = find(res_newton(it,:)<1e-12,1,'first');
    if ~isempty(ll_aaa)
        k_aaa(it) = ll_aaa;
    else
        k_aaa(it) = inf;
    end
    if ~isempty(ll_newton)
        k_newton(it) = ll_newton;
    else
        k_newton(it) = inf;
    end
    ritz_aaa(loc) = [];
    res_aaa(loc,:) = [];
end
[k_newton,locc] = sort(k_newton);
x_ritz = real(ritz_newton(locc(1:6)));
y_ritz = imag(ritz_newton(locc(1:6)));
k_lb = k_newton(1:6)';
k_aaa = k_aaa(locc(1:6))';

figure
plot(x_ritz,y_ritz,'x')
title('Six converged ($< 10^{-12}$) Ritz values of the problem','Interpreter','Latex')
xlim([0,300])
ylim([0,10])
xlabel('Re($\lambda$)','Interpreter','Latex')
ylabel('Im($\lambda$)','Interpreter','Latex')

figure
bar([k_lb k_aaa])
ylim([0,50])
legend('Leja-Bagby','AAA')
title('Number of iterations $< 10^{-12}$','Interpreter','Latex')
