clear
close
clc
rng(1)
addpath(genpath('src'))

% ===========
% Gun example
% ===========
%%%%%%%%%%%%%%
get_approx = 1; % Set to one to see approximation error (takes a long time to compute)
%%%%%%%%%%%%%%
% Example of the 'gun' problem of the 'nlevp' collection ()
A = gun();
% We will look for approximations of the eigenvalues with accuracy 10^{-12}
tol = 1e-13;
% Domain of interest is a half cicle in the complex plain with middle point
% 250^2 and radius 300^2-200^2, in and on which we make a large test set
m = 250^2; r = 300^2-200^2;
n_test = 1e3;
% Set test for AAA
set_aaa = m-r+2*r*rand(n_test,1)+1i*(rand(n_test,1)*r + sqrt(tol));
set_aaa(imag(set_aaa) > sin(acos((real(set_aaa)-m)/r))*r - sqrt(tol)) = [];
set_aaa = [[linspace(m-r+1e-2,m+r-1e-2,n_test/4)';m-r+2*r*(exp(1i*linspace(0,pi,n_test/4)')/2+.5)]; ...
    set_aaa(1:n_test/2)];
% Set test for testing error
n_test = 1e3;
set_test = m-r+2*r*rand(n_test,1)+1i*(rand(n_test,1)*r + sqrt(tol));
set_test(imag(set_test) > sin(acos((real(set_test)-m)/r))*r - sqrt(tol)) = [];
set_test = [[m-r+1e-2 + rand(n_test/4,1)*(m+r-1e-2 - (m-r+1e-2));m-r+2*r*(exp(1i*rand(n_test/4,1)*pi)/2+.5)]; ...
    set_test(1:n_test/2)];
% Bound set for determening lb points
n_lb = 1e5;
set_test_bound = [m-r,linspace(m-r+1e-2,m+r-1e-2,n_test/4),m-r+2*r*(exp(1i*linspace(0,pi,round(pi/2*n_lb)+2))/2+.5)].';
% Sample set for the poles for the Leja-Bagby points
Xi = -logspace(-8,8,n_lb) + 108.8774^2;
% Check the error on the test set for an increasing degree of interpolating
% polynomial, for aaa, set valued aaa and Newton Leja-Bagby
f_test = [A.f{1}(set_test),A.f{2}(set_test)];
f_test_n = [norm(f_test(:,1)),norm(f_test(:,2))];
err_aaa = zeros(1,32); err_aaa_sv = zeros(1,32); err_newton = zeros(1,32);
err_aaa_m = zeros(1,32); err_aaa_sv_m = zeros(1,32); err_newton_m = zeros(1,32);
np_aaa = zeros(1,32); np_aaa_sv = zeros(1,32);
if get_approx
    for d = 1:32
        [r_aaa_1,pol_1] = aaa_sv(A.f{1},set_aaa,'tol',tol,'mmax',d);
        [r_aaa_2,pol_2] = aaa_sv(A.f{2},set_aaa,'tol',tol,'mmax',d);
        [r_aaa_sv,pol_sv] = aaa_sv(@(s) [A.f{1}(s),A.f{2}(s)],set_aaa,'tol',tol,'mmax',d);
        r_newton = newton_lb(@(s) [A.f{1}(s),A.f{2}(s)],set_test_bound,Xi,d);
        
        np_aaa(d) = length(pol_1) + length(pol_2);
        np_aaa_sv(d) = length(pol_sv);
        err_aaa(d) = norm(sum(([r_aaa_1(set_test),r_aaa_2(set_test)]-f_test)/f_test_n,2));
        err_aaa_sv(d) = norm(sum((r_aaa_sv(set_test)-f_test)/f_test_n,2));
        err_newton(d) = norm(sum((r_newton(set_test)-f_test)/f_test_n,2));
        
        % Matrix error:
        err_aaa_m_i = zeros(1,length(set_test));
        err_aaa_sv_m_i = zeros(1,length(set_test));
        err_newton_m_i = zeros(1,length(set_test));
        ff1 = r_aaa_sv(set_test);
        ff2 = r_newton(set_test);
        for it = 1:length(set_test)
            err = A.f{1}(set_test)-ff1;
            AA = A.C{1}-set_test(it)*A.C{2} + A.f{1}(set_test(it))*A.C{3} + A.f{2}(set_test(it))*A.C{4};
            AA_aaa = A.C{1}-set_test(it)*A.C{2} + r_aaa_1(set_test(it))*A.C{3} + r_aaa_2(set_test(it))*A.C{4};
            AA_aaa_sv = A.C{1}-set_test(it)*A.C{2} + ff1(it,1)*A.C{3} + ff1(it,2)*A.C{4};
            AA_newton = A.C{1}-set_test(it)*A.C{2} + ff2(it,1)*A.C{3} + ff2(it,2)*A.C{4};
            
            err_aaa_m_i(it) = norm((AA_aaa-AA),1)/norm(AA,1);
            err_aaa_sv_m_i(it) = norm((AA_aaa_sv-AA),1)/norm(AA,1);
            err_newton_m_i(it) = norm((AA_newton-AA),1)/norm(AA,1);
        end
        err_aaa_m(d) = norm(err_aaa_m_i);
        err_aaa_sv_m(d) = norm(err_aaa_sv_m_i);
        err_newton_m(d) = norm(err_newton_m_i);
    end
end
% Get the eigenvalues of the non-linear matrix function
v0 = rand(A.n,1) + 1i*rand(A.n,1);
opts = struct('k',10,'m',100,'shifts',r*[2/3,(1+1i)/3,0,(-1+1i)/3,-2/3] + m,'v0',v0,'tol',tol,'tol_appr',tol);
[~,~,~,info_aaa] = eign(A,set_aaa,opts);
opts.kind = 'ratNewton';
opts.d = 30;
opts.Xi = Xi;
[~,~,~,info_Newton] = eign(A,set_test_bound,opts);

% Postprocessing of the residues and Ritz values
[ lambda_sor,res_sor_newton ] = sort_eig( info_Newton.Lambda,info_Newton.Res );
res_sor_newton(res_sor_newton==0) = inf;
is_in = logical((real(lambda_sor(:,end))>=min(real(opts.shifts))).*(real(lambda_sor(:,end))<=max(real(opts.shifts))));
[ ~,inds_newton ] = sort(res_sor_newton(is_in,50));
res_sor_newton = res_sor_newton(is_in,:);
[ lambda_sor,res_sor_aaa ] = sort_eig( info_aaa.Lambda,info_aaa.Res );
res_sor_aaa(res_sor_aaa==0) = inf;
is_in = logical((real(lambda_sor(:,end))>=min(real(opts.shifts))).*(real(lambda_sor(:,end))<=max(real(opts.shifts))));
[ ~,inds_aaa ] = sort(res_sor_aaa(is_in,50));
ritz = lambda_sor(is_in,end);
ritz = ritz(inds_aaa(1:5));
res_sor_aaa = res_sor_aaa(is_in,:);

% Make figures
figure
plot(real(sqrt(ritz)),imag(sqrt(ritz)),'x','color','red')
pol_newton = info_Newton.pol(real(info_Newton.pol)>0);
pol_aaa = info_aaa.pol(real(info_aaa.pol)>0);
hold on
plot(real(sqrt(pol_newton)),imag(sqrt(pol_newton)),'d','color',[0,0.5,0])
plot(real(sqrt(pol_aaa)),imag(sqrt(pol_aaa)),'o','color',[0,0,0.5])
plot(real(sqrt(set_test_bound)),imag(sqrt(set_test_bound)),'-k')
xlim([0,350])
ylim([-50,150])
xlabel('Re $\lambda$','Interpreter','Latex','Fontsize',12)
ylabel('Im $\lambda$','Interpreter','Latex','Fontsize',12)
legend('Ritz values','Leja-Bagby poles','AAA poles','Location','NorthWest')
title('Residues','Fontsize',14,'Interpreter','Latex')

figure
semilogy(1:32,err_newton,'-d','color',[0,0.5,0])
hold on
semilogy(np_aaa,err_aaa,'-s','color',[0,0.5,0.5])
semilogy(np_aaa_sv,err_aaa_sv,'-o','color',[0,0,0.5])
semilogy(1:32,err_newton_m,'--','color',[0,0.5,0])
semilogy(np_aaa,err_aaa_m,'--','color',[0,0.5,0.5])
semilogy(np_aaa_sv,err_aaa_sv_m,'--','color',[0,0,0.5])
xlabel('number of poles ($m$)','Interpreter','Latex','Fontsize',12)
ylabel('error ($E$)','Interpreter','Latex','Fontsize',12)
legend('Newton Leja-Bagby','AAA','Set-valued AAA')
title('Approximation error','Fontsize',14,'Interpreter','Latex')

figure
semilogy(res_sor_newton(inds_newton(1:5),:).','-','color',[0,0.5,0])
hold on
semilogy(res_sor_aaa(inds_aaa(1:5),:).','-','color',[0,0,0.5])
ylim([1e-16,1])
xlabel('iteration $j$','Interpreter','Latex','Fontsize',12)
ylabel('$R_x$','Interpreter','Latex','Fontsize',12)
legend('Newton Leja-Bagby','Set-valued AAA')
title('Residues','Fontsize',14,'Interpreter','Latex')