function A = beam()
%BEAM - Model of a sandwich beam
%
% See: V AN B EEUMEN , R., M EERBERGEN , K. & M ICHIELS , W. (2013) 
% A rational Krylov method based on Hermite interpolation for nonlinear eigenvalue problems. 
% SIAM Journal on Scientific Computing, 35, A327–A350.
beam = load('beam.mat');

A.C = {beam.K,beam.M,beam.C};
A.p = [1,0,-1];
A.f{1} = @(x) bfun(x);

    function f = bfun(s)
        g0    = 3.504e5;  % Static shear modulus
        gInf  = 3.062e9;  % Asymptotic shear modulus
        tau   = 8.230e-9; % relaxation time
        alpha = 0.675;    % fractional parameter
        
        f = (g0 + gInf*(1i*s*tau).^alpha)./(1 + (1i*s*tau).^alpha);
    end
end